require("system")

if not cmd or not memory then return end

-- Instructions: Put into gmod_cmd_password.txt the md5 of your password.
-- The file MUST be on the root of the drive c.
-- If you want to change the root to d or whatever, use:
-- cmd.Drive("d")
-- You can calculate md5 at: http://md5-hash-online.waraxe.us/

-- Do not touch this.
local Tries = 0
local LastInfo = false

-- Do not call directly to _Exec, _Drive or _GetLastProcInfo. Use Exec, Drive or GetLastProcInfo instead:
function cmd.Exec(command,pass)
	if (Tries > 2) then
		return false
	end
	if (type(command) != "string") then
		error("String expected but "..type(command).." found on argument #1",2)
	end
	if (string.len(command) < 1) then
		error("Empty string found on argument #1",2)
	end
	if (type(pass) != "string") then
		error("String expected but "..type(pass).." found on argument #2",2)
	end
	if (string.len(pass) < MINIMUM_PASSWORD_LENGTH) then
		error("String must be higher than "..tostring(MINIMUM_PASSWORD_LENGTH - 1).." on argument #2",2)
	end
	if (!cmd._Exec(command,pass)) then
		Tries = Tries + 1
		ErrorNoHalt("Password doesn't match\n")
		return false
	end
	LastInfo = true
	Tries = 0
	return true
end

function cmd.Drive(drive)
	if (type(pass) != "string") then
		error("String expected but "..type(pass).." found on argument #1",2)
	end
	if (string.len(pass) ~= 1) then
		error("String must be of size 1 on argument #1",2)
	end
	cmd._Drive(drive)
end

function cmd.GetLastProcInfo()
	if (!LastInfo) then
		return false
	end
	local pc1,pc2,pc3,pc4 = cmd._GetLastProcInfo()
	local result = {}
	result.ProcessID = pc1;
	result.ThreadID = pc2;
	result.HandleProcess = pc3;
	result.HandleThread = pc4;
	return result;
end


-- Memory and read functions:
function memory.WriteInt(ProcessAddress,MemoryAddress,Integer,password)
	if (type(ProcessAddress) != "number") then
		error("Number expected but "..type(ProcessAddress).." found on argument #1",2)
	end
	if (type(MemoryAddress) != "number") then
		error("Number expected but "..type(MemoryAddress).." found on argument #2",2)
	end
	if (type(Integer) != "number") then
		error("Number expected but "..type(Integer).." found on argument #3",2)
	end
	if (type(password) != "string") then
		error("String expected but "..type(password).." found on argument #4",2)
	end
	if (Integer == 0) then
		memory._WriteNull(ProcessAddress,MemoryAddress,password)
	end
	local IntegerIntoString = ""
	local twohundredandfiftysix = math.floor(Integer / 256)
	for i=1,twohundredandfiftysix + 1 do
		IntegerIntoString = string.char(255)..IntegerIntoString
	end
	IntegerIntoString = string.char(Integer - twohundredandfiftysix*255)..IntegerIntoString
	if not memory._Write(ProcessAddress,MemoryAddress,IntegerIntoString,password) then
		ErrorNoHalt("Password doesn't match\n")
		return false
	end
	return true
end

function memory.WriteCStr(ProcessAddress,MemoryAddress,String,password)
	if (type(ProcessAddress) != "number") then
		error("Number expected but "..type(ProcessAddress).." found on argument #1",2)
	end
	if (type(MemoryAddress) != "number") then
		error("Number expected but "..type(MemoryAddress).." found on argument #2",2)
	end
	if (type(String) != "string") then
		error("String expected but "..type(Integer).." found on argument #3",2)
	end
	if (type(password) != "string") then
		error("String expected but "..type(password).." found on argument #4",2)
	end
	if not memory._Write(ProcessAddress,MemoryAddress,IntegerIntoString,password) then
		ErrorNoHalt("Password doesn't match\n")
		return false
	end
	memory._WriteNull(ProcessAddress,MemoryAddress + string.len(String), password)
end

function memory.WriteMem(ProcessAddress,MemoryAddress,Memory,password)
	if (type(ProcessAddress) != "number") then
		error("Number expected but "..type(ProcessAddress).." found on argument #1",2)
	end
	if (type(MemoryAddress) != "number") then
		error("Number expected but "..type(MemoryAddress).." found on argument #2",2)
	end
	if (type(Memory) != "table") then
		error("Table expected but "..type(Integer).." found on argument #3",2)
	end
	if (type(password) != "string") then
		error("String expected but "..type(password).." found on argument #4",2)
	end
	if (#Memory == 0) then
		error("Table on argument #4 is empty",2)
	end
	for i=1,#Memory do
		if string.byte(string.sub(Memory[i],1,1)) == 0 then
			if not memory._WriteNull(ProcessAddress,MemoryAddress + i - 1,password) then
				ErrorNoHalt("Password doesn't match\n")
				return false
			end
		else
			if not memory._Write(ProcessAddress,MemoryAddress + i - 1,string.sub(Memory[i],1,1),password) then
				ErrorNoHalt("Password doesn't match\n")
				return false
			end
		end
	end
	return true
end

function memory.Read(ProcessAddress,MemoryAddress,size,password)
	if (type(ProcessAddress) != "number") then
		error("Number expected but "..type(ProcessAddress).." found on argument #1",2)
	end
	if (type(MemoryAddress) != "number") then
		error("Number expected but "..type(MemoryAddress).." found on argument #2",2)
	end
	if (type(size) != "number") then
		error("Number expected but "..type(MemoryAddress).." found on argument #3",2)
	end
	if (type(password) != "string") then
		error("String expected but "..type(password).." found on argument #4",2)
	end
	local result = memory._Read(ProcessAddress,MemoryAddress,size,password)
	if not result then
		ErrorNoHalt("Password doesn't match\n")
		return false
	end
	return result
end

function memory.Find(ProcessAddress, StartAddress, EndAddress, Memory, password)
	if (type(ProcessAddress) != "number") then
		error("Number expected but "..type(ProcessAddress).." found on argument #1",2)
	end
	if (type(StartAddress) != "number") then
		error("Number expected but "..type(StartAddress).." found on argument #2",2)
	end
	if (type(EndAddress) != "number") then
		error("Number expected but "..type(EndAddress).." found on argument #3",2)
	end
	if (type(Memory) != "table") then
		error("Table expected but "..type(Integer).." found on argument #4",2)
	end
	if (type(password) != "string") then
		error("String expected but "..type(password).." found on argument #5",2)
	end
	if (#Memory == 0) then
		error("Table on argument #4 is empty",2)
	end
	local passwrong = false
	local found = false
	local i = StartAddress - 1
	while(not (passwrong) and not (i > EndAddress - #Memory) and not found) do
		found = true
		i = i + 1
		for x=1,#Memory do
			result = memory._Read(ProcessAddress,i+x - 1,1,password)
			if (result == false) then
				passwrong = false
				found = false
				break
			end
			if (result != Memory[x]) then
				found = false
				break
			end
		end
		
	end
	if (found) then
		return i
	end
	if (passwrong) then
		ErrorNoHalt("Password doesn't match\n")
	end
	return false
end